#include <stdio.h>
#include <errno.h>
#include <crypto.h>
#if SSLEAY_VERSION_NUMBER >= 0x0090581fL
#if SSLEAY_VERSION_NUMBER < 0x0090600fL
#define OPENSSL_malloc Malloc
#endif
/* Only compile this is running with OpenSSL-0.9.5 or greater */
#include <bio.h>

static int bin_write(BIO *b, char *buf,int num);
static int bin_puts(BIO *b, char *str);
static long bin_ctrl(BIO *b,int cmd,long arg1,char *arg2);
static int bin_new(BIO *b);
static int bin_free(BIO *b);
static long bin_callback_ctrl(BIO *b,int cmd,void (*cb)());

typedef struct bin_struct
	{
	int where;         /* doing indent (0) or user output (1) */
	int ind_len;       /* how big is indent */
	int ind_off;       /* where in offset are we */
	int buf_len;       /* how big is buffer */
	int buf_off;       /* where in buffer are we */
	char  *buf_prefix; /* the prefix string which produces the indent */
	} BIO_BIN_CTX;

static BIO_METHOD methods_bin=
	{
    BIO_TYPE_FILTER,"indent output",
    bin_write,
    NULL, /* bin_read, */
    bin_puts, 
    NULL, /* bin_gets, */
    bin_ctrl,
    bin_new,
    bin_free,
    bin_callback_ctrl,
	};

BIO_METHOD *BIO_f_indent(void)
    {
    return(&methods_bin);
    }

static int bin_new(BIO *b)
	{
	BIO_BIN_CTX *ctx;
    ctx=(BIO_BIN_CTX *)OPENSSL_malloc(sizeof(BIO_BIN_CTX));
    if (ctx == NULL) return(0);

	ctx->where = 0;
	ctx->ind_off = 0;
	ctx->ind_len = 4;
	ctx->buf_off = 0;
	ctx->buf_len = 0;
	ctx->buf_prefix = (char *)OPENSSL_malloc(ctx->ind_len);
	if (ctx->buf_prefix == NULL) return(0);
	memset(ctx->buf_prefix,' ',ctx->ind_len);

	b->init=1;
	b->ptr=(char *)ctx;
	b->flags=0;
	
	return(1);
	}

static int bin_free(BIO *b)
	{
	BIO_BIN_CTX * ctx;

	if (b == NULL) return(0);
	ctx=(BIO_BIN_CTX *)b->ptr;
	if (ctx) 
		{
		if (ctx->buf_prefix)
			OPENSSL_free(ctx->buf_prefix);
		OPENSSL_free(ctx);
		b->ptr=NULL;
		}
	b->init = 0;
	b->flags = 0;
	return(1);
	}

static int bin_write(BIO *b, char *in, int inl)
	{
	BIO_BIN_CTX *ctx;
	int n;
	int i;
	int j;
	int ret;
	int NNN;

    ctx=(BIO_BIN_CTX *)b->ptr;
    BIO_clear_retry_flags(b);
	
	ctx->buf_len = inl;
	while(1) 
		{
		if (ctx->where == 0) 
			{
			n=ctx->ind_len - ctx->ind_off;
			while( n > 0)
				{
				i = BIO_write(b->next_bio,
						&(ctx->buf_prefix[ctx->ind_off]),n);
				if (i <= 0)
					{
					BIO_copy_next_retry(b);
					return(i);
					}
				ctx->ind_off += i;
				n -= i;
				}
			ctx->where = 1;
			ctx->ind_off = 0;
			}
	
		if ((in == NULL) || (inl <= 0)) 
			{
			ctx->buf_off = 0;
			return(0);
			}
	
		j = ctx->buf_len - ctx->buf_off;
		n = 0;
		while (n < j)
			{
			if (in[ctx->buf_off + n++] == '\n')
				break;
			}
		
		while (n > 0)
			{
			i = BIO_write(b->next_bio,&(in[ctx->buf_off]),n);	
			if (i <= 0)
				{
				BIO_copy_next_retry(b);
				return(i);
				}
			if ( (i == n) && in[ctx->buf_off + i - 1] == '\n')
					ctx->where = 0;
			ctx->buf_off +=i;
			n-=i;	
			}

		if(ctx->buf_off == ctx->buf_len)
			{
			ret = ctx->buf_off;
			ctx->buf_off = 0;

			return ret;
			}
		}
	}
	
static bin_puts(BIO *b, char * str)
	{
		return bin_write(b, str, strlen(str));

	}

static long bin_ctrl(BIO *b, int cmd, long num, char *ptr)
	{
	BIO_BIN_CTX *ctx;
	 long ret=1;
	int i;

	ctx=(BIO_BIN_CTX *)b->ptr;
	switch (cmd)
		{
		case BIO_CTRL_SET_FILENAME:
			ctx->ind_off = 0;
			ctx->ind_len =  num ;
			if (ctx->buf_prefix != NULL) 
				OPENSSL_free(ctx->buf_prefix);
			ctx->buf_prefix = (char*)OPENSSL_malloc(ctx->ind_len);
			if (ctx->buf_prefix == NULL) return(0);
			memcpy(ctx->buf_prefix,ptr,ctx->ind_len);
			ret = 1;
			break;
			
		case BIO_CTRL_RESET:
			ctx->where = 0;
			ctx->ind_off = 0;
			ctx->ind_len = 4;
			ctx->buf_off = 0;
			ctx->buf_len = 0;
			ret=BIO_ctrl(b->next_bio,cmd,num,ptr);
			break;
		case BIO_C_DO_STATE_MACHINE:
			BIO_clear_retry_flags(b);
			ret=BIO_ctrl(b->next_bio,cmd,num,ptr);
			BIO_copy_next_retry(b);
			break;
		default:
			ret=BIO_ctrl(b->next_bio,cmd,num,ptr);
			break;
		}
	return(ret);
	}

static long bin_callback_ctrl(BIO *b, int cmd, void (*fp)())
	{
	long ret=1;

	if (b->next_bio == NULL) return(0);
	switch (cmd)
		{
		default:
			ret=BIO_callback_ctrl(b->next_bio,cmd,fp);
			break;
		}
	return(ret);
	}
#endif /* OPenSSL-0.9.5 and above only */
